/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "find.h"

void openldev::populate_find_toolbar()
{
  // Create Horizontal Find 'Toolbar'
  hbox_find = new Gtk::HBox(false, 0);
  label_find = new Gtk::Label("Find: ");
  label_error = new Gtk::Label("Text not found!");
  find_text = new Gtk::Entry();
  image_close = new Gtk::Image(Gtk::StockID("gtk-stop"), Gtk::IconSize(1));
  button_find = new Gtk::Button("Find Next");
  button_find_prev = new Gtk::Button("Find Previous");
  button_close = new Gtk::Button();
  image_error = new Gtk::Image(Gtk::StockID("gtk-dialog-error"), Gtk::IconSize(1));
  
	button_close->set_relief(Gtk::RELIEF_NONE);
	button_close->add(*image_close);
  button_find->set_relief(Gtk::RELIEF_NONE);
  button_find_prev->set_relief(Gtk::RELIEF_NONE);
  
  hbox_find->set_spacing(5);
  hbox_find->set_border_width(3);
  hbox_find->pack_start(*button_close, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*label_find, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*find_text, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*button_find, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*button_find_prev, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*image_error, Gtk::PACK_SHRINK);
  hbox_find->pack_start(*label_error, Gtk::PACK_SHRINK);
  
  button_close->signal_clicked().connect(sigc::mem_fun(*this, &openldev::on_edit_find));
  button_find->signal_clicked().connect(sigc::mem_fun(*this, &openldev::on_find_next));
  button_find_prev->signal_clicked().connect(sigc::mem_fun(*this, &openldev::on_find_previous));
}

// Run the find dialog & call find_next to highlight text
void openldev::on_edit_find() 
{
  if (hbox_find->is_visible())
    hbox_find->hide();
  else
  {
    hbox_find->show();
    if (hbox_goto->is_visible())
      on_edit_goto_line();
  }
}

// Find & highlight the text specified by the user
void openldev::on_find_next()
{
  // Check if there was no text specified
  if (!find_text->get_text().empty())
  {
    Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
    Gtk::TextIter iter, end, enditer = buffer->end();
  
    // If there is no selected text, then search from the cursor position
    if (buffer->get_selection_bound()->get_iter() > buffer->get_insert()->get_iter())
      iter = buffer->get_selection_bound()->get_iter();
    else
      iter = buffer->get_insert()->get_iter();
  
    // If it is found, select the text
    if (iter.forward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
      mark_found_text(iter, end);
    // Throw an error if it cannot be found
    else 
    {
      iter = buffer->begin();
      if (iter.forward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,enditer)) 
        mark_found_text(iter, end);
      else
      {
        label_error->show();
        image_error->show();
      }
    }
  }
}

// Find & highlight the text specified by the user
void openldev::on_find_previous()
{
  // Check if there was no text specified
  if (!find_text->get_text().empty())
  {
    Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
    Gtk::TextIter iter, end, beginiter = buffer->begin();
  
    // If there is no selected text, then search from the cursor position
    if (buffer->get_insert()->get_iter() > buffer->get_selection_bound()->get_iter())
      iter = buffer->get_selection_bound()->get_iter();
    else
      iter = buffer->get_insert()->get_iter();
  
    // If it is found, select the text
    if (iter.backward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,beginiter)) 
      mark_found_text(iter, end);
    // Throw an error if it cannot be found
    else 
    {
      iter = buffer->end();
      if (iter.backward_search(find_text->get_text(), Gtk::TEXT_SEARCH_VISIBLE_ONLY,iter,end,beginiter)) 
        mark_found_text(iter, end);
      else
      {
        label_error->show();
        image_error->show();
      }
    }
  }
}

void openldev::mark_found_text(Gtk::TextIter iter, Gtk::TextIter end)
{
  Glib::RefPtr<Gtk::TextBuffer> buffer = noteman->get_current_textview()->get_buffer();
  buffer->move_mark(buffer->get_insert(), iter);
  buffer->move_mark(buffer->get_selection_bound(), end);
  noteman->scroll_to_mark();
  set_focus(*noteman->get_current_textview());
      
  label_error->hide();
  image_error->hide();
}
